package com.catkissfish.springboot.mocikito.demo;

import com.catkissfish.springboot.mocikito.demo.dto.UserUpdateReqDto;
import com.catkissfish.springboot.mocikito.demo.service.UserService;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.ArgumentMatchers;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.Spy;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.mockito.stubbing.Answer;

import java.util.ArrayList;

import static org.mockito.Mockito.*;

@ExtendWith(MockitoExtension.class)
public class Demo03 {
    @Mock
    private UserService userService;
    @Mock
    private ArrayList<String> mockList;
    @Spy
    private ArrayList<String> spyList;

    /**
     * 指定返回值
     */
    @Test
    public void test1(){
        Mockito.doReturn(1).when(userService).modifyById(ArgumentMatchers.any(UserUpdateReqDto.class));
//        Assertions.assertEquals(0, userService.modifyById(new UserUpdateReqDto()));
        Assertions.assertEquals(1, userService.modifyById(new UserUpdateReqDto()));
    }

    /**
     * void返回值方法插桩
     */
    @Test
    public void test2(){
        doNothing().when(mockList).clear();
        mockList.clear();
        //验证调用一次
        verify(mockList, times(1)).clear();
    }

    /**
     * 抛出异常
     */
    @Test
    public void test3(){
        doThrow(RuntimeException.class).when(mockList).get(anyInt());
        try {
            mockList.get(5);
        }catch (Exception e) {
            Assertions.assertTrue(e instanceof RuntimeException);
        }
    }

    /**
     * 多次插桩
     */
    @Test
    public void test4(){
        doReturn(1).doReturn(2).doReturn(3).when(mockList).size();
        Assertions.assertEquals(1, mockList.size());
        Assertions.assertEquals(2, mockList.size());
        Assertions.assertEquals(3, mockList.size());
        Assertions.assertEquals(4, mockList.size());
    }

    /**
     * 实现指定的逻辑插桩
     */
    @Test
    public void test5(){
        doAnswer(new Answer<String>() {
            @Override
            public String answer(InvocationOnMock invocationOnMock) throws Throwable {
                Integer argument = invocationOnMock.getArgument(0, Integer.class);
                return String.valueOf(argument * 100);
            }
        }).when(mockList).get(anyInt());
        String result = mockList.get(3);
        Assertions.assertEquals("300", result);
    }

    /**
     * 执行真正的原始方法
     */
    @Test
    public void test6(){
//        //对mock对象插桩执行原始方法
//        when(mockList.get(0)).thenCallRealMethod();
//        System.out.println(mockList.get(0));

        //未插桩spy对象默认就会调用真实方法。
        doReturn("666").when(spyList).get(0);
        System.out.println(spyList.get(0));
    }

    /**
     * verify的使用
     */
    @Test
    public void test7(){
        mockList.clear();
        //从未调用过clear方法
//        verify(mockList, never()).clear();
        //至少调用过多少次
        verify(mockList, atLeast(1)).clear();
    }
}
